IntroWindow = class(Turbine.UI.Lotro.Window)
function IntroWindow:Constructor(handler, inProgress)
    Turbine.UI.Lotro.Window.Constructor(self)

    self.width = 1000
    self.height = 700
    self.handler = handler
    self.inProgress = inProgress
    self:SetSize(self.width, self.height)
    self:SetText("Hardcore")
    self:SetPosition(Turbine.UI.Display.GetWidth() / 2 - self.width / 2, Turbine.UI.Display.GetHeight() / 2 - self.height / 2)
    self:SetVisible(not(self.inProgress))

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.partyNames = {}
    self.achievements = {}

    self.optionHeaders = {
        [1] = "Rules",
        [2] = "Fellowship",
        [3] = "Achievements",
    }

    self.extraOptionsHeaders = {
        [1] = "Level log",
        [2] = "Stats",
        [3] = "Verification",
    }

    self.classObjectsToName = {
        [Turbine.Gameplay.Class.Burglar] = "Burglar",
        [Turbine.Gameplay.Class.Captain] = "Captain",
        [Turbine.Gameplay.Class.Champion] = "Champion",
        [Turbine.Gameplay.Class.Mariner] = "Mariner",
        [Turbine.Gameplay.Class.Guardian] = "Guardian",
        [Turbine.Gameplay.Class.Hunter] = "Hunter",
        [Turbine.Gameplay.Class.LoreMaster] = "LoreMaster",
        [Turbine.Gameplay.Class.Minstrel] = "Minstrel",
        [Turbine.Gameplay.Class.RuneKeeper] = "Runekeeper",
        [Turbine.Gameplay.Class.Warden] = "Warden",
        [Turbine.Gameplay.Class.Beorning] = "Beorning",
        [Turbine.Gameplay.Class.Brawler] = "Brawler",
        [Turbine.Gameplay.Class.BlackArrow] = "BlackArrow",
        [Turbine.Gameplay.Class.Defiler] = "Defiler",
        [Turbine.Gameplay.Class.Reaver] = "Reaver",
        [Turbine.Gameplay.Class.Stalker] = "Stalker",
        [Turbine.Gameplay.Class.WarLeader] = "WarLeader",
        [Turbine.Gameplay.Class.Weaver] = "Weaver",
    } 

    self.optionListBox = Turbine.UI.ListBox()
    self.optionListBox:SetParent(self)
    self.optionListBox:SetSize(260 * 3, 30)
    self.optionListBox:SetPosition(self:GetWidth() / 2 -  self.optionListBox:GetWidth() / 2, 40)
    --self.optionListBox:SetBackColor(Turbine.UI.Color.Red)
    self.optionListBox:SetOrientation(Turbine.UI.Orientation.Horizontal)

    self.extraOptionListBox = Turbine.UI.ListBox()
    self.extraOptionListBox:SetParent(self)
    self.extraOptionListBox:SetSize(260 * 3, 30)
    self.extraOptionListBox:SetPosition(self:GetWidth() / 2 -  self.extraOptionListBox:GetWidth() / 2, 70)
    --self.extraOptionListBox:SetBackColor(Turbine.UI.Color.Red)
    self.extraOptionListBox:SetOrientation(Turbine.UI.Orientation.Horizontal)

    self.optionPages = {}


    for key, value in ipairs(self.optionHeaders) do
        local header = Header()
        header:SetText(value)
        header.BoxHeaderLabel.MouseClick = function()
            self:SwapPages(value)
        end
        self.optionListBox:AddItem(header)

        local optionControl = Turbine.UI.Control()
        optionControl:SetParent(self)
        optionControl:SetSize(self.optionListBox:GetWidth(), 550)
        optionControl:SetPosition(self:GetWidth() / 2 -  self.optionListBox:GetWidth() / 2, 40 + 30 + 30)
       -- optionControl:SetBackColor(Turbine.UI.Color.Blue)

        self.optionPages[value] = optionControl
    end

    if self.inProgress then
        for key, value in ipairs(self.extraOptionsHeaders) do
            local header = Header()
            header:SetText(value)
            header.BoxHeaderLabel.MouseClick = function()
                self:SwapPages(value)
            end
            self.extraOptionListBox:AddItem(header)

            local optionControl = Turbine.UI.Control()
            optionControl:SetParent(self)
            optionControl:SetSize(self.optionListBox:GetWidth(), 550)
            optionControl:SetPosition(self:GetWidth() / 2 -  self.optionListBox:GetWidth() / 2, 40 + 30 + 30)
            --optionControl:SetBackColor(Turbine.UI.Color.Blue)

            self.optionPages[value] = optionControl
        end
    end

    self.rulesLabel = self:GetRulesControl(self.optionPages["Rules"])

    --[[
    self.rulesLabel = Turbine.UI.Label()
    self.rulesLabel:SetParent(self.optionPages["Rules"])
    self.rulesLabel:SetSize(self.optionListBox:GetWidth(), 550)
    self.rulesLabel:SetBackColor(Turbine.UI.Color.Blue)
    self.rulesLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.rulesLabel:SetOutlineColor(Turbine.UI.Color.Black)
    self.rulesLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiqua16)
    self.rulesLabel:SetText(GetRulesText())

    self.rulesScrollbar = Turbine.UI.Lotro.ScrollBar()
    self.rulesScrollbar:SetParent(self.optionPages["Rules"])
    self.rulesScrollbar:SetSize(10, 550)
    self.rulesScrollbar:SetPosition(self.rulesLabel:GetLeft() + self.rulesLabel:GetWidth() - 10, 0)
    self.rulesScrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)

    self.rulesLabel:SetVerticalScrollBar(self.rulesScrollbar)
    --]]

    self.beginAdventureButton = Turbine.UI.Lotro.Button()
    self.beginAdventureButton:SetParent(self)
    self.beginAdventureButton:SetWidth(150)
    self.beginAdventureButton:SetPosition(self:GetWidth() - 250, self:GetHeight() - 50)
    self.beginAdventureButton:SetText("Begin Adventure")
    self.beginAdventureButton:SetVisible(not(self.inProgress))

    self.beginAdventureButton.Click = function()
        self.handler:Begin(self.achievements, self.partyNames)
    end

    
    if self.inProgress then
        self.optionPages["Stats"].VisibleChanged = function()
            if self.optionPages["Stats"]:IsVisible() then
                if self.statPage then
                    self.statPage:SetParent(nil)
                end
                self.statPage = nil
                self.statPage = StatControl(self.handler:GetCharacterData(), self.optionPages["Stats"], handler.startTime)
            end
        end

        self.optionPages["Level log"].VisibleChanged = function()
            if self.optionPages["Level log"]:IsVisible() then
                if self.questLogControl then
                    self.questLogControl:SetParent(nil)
                end
                self.questLogControl = nil
                self.questLogControl = QuestLogControl(self.handler:GetCharacterData(), self.optionPages["Level log"])
            end
        end

        self.verificationControl = self:GetVerificationControl(self.optionPages["Verification"])
    end

    self.partyControl = self:GetPartyControl(self.optionPages["Fellowship"])


    
    self:AchievementPage()
    self:SwapPages("Rules")
end

function IntroWindow:AchievementPage()
    self.achievementsListBox = Turbine.UI.ListBox()
    self.achievementsListBox:SetParent(self.optionPages["Achievements"])
    self.achievementsListBox:SetSize(self.achievementsListBox:GetParent():GetWidth(), self.achievementsListBox:GetParent():GetHeight())
    --self.achievementsListBox:SetBackColor(Turbine.UI.Color.Red)
    self.achievementsListBox:SetOrientation(Turbine.UI.Orientation.Horizontal)
    self.achievementsListBox:SetMaxColumns(1)

    self.achievementsScrollbar = Turbine.UI.Lotro.ScrollBar()
    self.achievementsScrollbar:SetParent(self.optionPages["Achievements"])
    self.achievementsScrollbar:SetSize(10, 550)
    self.achievementsScrollbar:SetPosition(self.rulesLabel:GetLeft() + self.rulesLabel:GetWidth() - 10, 0)
    self.achievementsScrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)
    self.achievementsScrollbar:SetSmallChange(1)
    self.achievementsScrollbar:SetLargeChange(2)

    self.achievementsListBox:SetVerticalScrollBar(self.achievementsScrollbar)

    self.optionPages["Achievements"].VisibleChanged = function()
        if self.optionPages["Achievements"]:IsVisible() then
            self.achievementsListBox:ClearItems()
            local achievementCategories = GetAchievementsCategories()
            local achievements = Achievements.GetAchievements()

            local className = self.classObjectsToName[self.player:GetClass()]

            for key, value in PrimeUtils.Utils.spairs(achievementCategories, function(t, a, b) return a < b end) do
                local categoryLabel = Turbine.UI.Label()
                categoryLabel:SetSize(self.achievementsListBox:GetWidth(), 60)
                categoryLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
                categoryLabel:SetOutlineColor(Turbine.UI.Color.Black)
                categoryLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold25)
                categoryLabel:SetTextAlignment(Turbine.UI.ContentAlignment.BottomLeft)
                categoryLabel:SetText(value)

                local subAchievementListBox = Turbine.UI.ListBox()
                subAchievementListBox:SetSize(self.achievementsListBox:GetParent():GetWidth(), 10)
                --subAchievementListBox:SetBackColor(Turbine.UI.Color.Green)
                subAchievementListBox:SetOrientation(Turbine.UI.Orientation.Horizontal)
                subAchievementListBox:SetMaxColumns(2)

                local items = false
                if self.inProgress then
                    for achievName, ID in PrimeUtils.Utils.spairs(self.handler:GetAchievements(), function(t, a, b) return a < b end) do
                        local tableValue = achievements[ID]
                        local class
                        if key > 2 then
                            class = false
                        end
                        if tableValue then
                            if tableValue.Class then
                                class = false
                                for _, achievementClassName in pairs(tableValue.Class) do
                                    if className == achievementClassName then
                                        class = true
                                    end
                                end
                            end
                            if tableValue["Category"] == key then
                                items = true
                                subAchievementListBox:AddItem(self:GetAchievementControl(ID, achievements[ID], not(self.inProgress), class))
                            end
                        end
                    end
                else 
                    for ID, tableValue in PrimeUtils.Utils.spairs(achievements, function(t, a, b) return a < b end) do
                        local class
                        if key > 2 then
                            class = false
                        end
                        if tableValue.Class then
                            class = false
                            for _, achievementClassName in pairs(tableValue.Class) do
                                if className == achievementClassName then
                                    class = true
                                end
                            end
                        end
                        if tableValue["Category"] == key then
                            items = true
                            subAchievementListBox:AddItem(self:GetAchievementControl(ID, achievements[ID], not(self.inProgress), class))
                        end
                    end
                end

                subAchievementListBox:SetSize(self.optionListBox:GetParent():GetWidth(), math.ceil(subAchievementListBox:GetItemCount() / 2) * 80)--)

                subAchievementListBox.PositionChanged = function()
                    for i = 1, subAchievementListBox:GetItemCount() do
                        subAchievementListBox:GetItem(i):CheckVisibility(subAchievementListBox:GetTop())
                    end
                end                

                if items then
                    self.achievementsListBox:AddItem(categoryLabel)
                    self.achievementsListBox:AddItem(subAchievementListBox)
                end
            end

            if self.inProgress then
                local categoryLabel = Turbine.UI.Label()
                categoryLabel:SetSize(self.achievementsListBox:GetWidth(), 60)
                categoryLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
                categoryLabel:SetOutlineColor(Turbine.UI.Color.Black)
                categoryLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold25)
                categoryLabel:SetTextAlignment(Turbine.UI.ContentAlignment.BottomLeft)
                categoryLabel:SetText("Completed")

                local subAchievementListBox = Turbine.UI.ListBox()
                subAchievementListBox:SetSize(self.achievementsListBox:GetParent():GetWidth(), 10)
                --subAchievementListBox:SetBackColor(Turbine.UI.Color.Green)
                subAchievementListBox:SetOrientation(Turbine.UI.Orientation.Horizontal)
                subAchievementListBox:SetMaxColumns(2)

                local items = false
                for ID, _ in pairs(self.handler:GetAchievementsCompleted()) do
                    items = true
                    subAchievementListBox:AddItem(self:GetAchievementControl(ID, achievements[ID], not(self.inProgress), nil, true))
                end

                if items then
                    subAchievementListBox:SetSize(self.optionListBox:GetParent():GetWidth(), math.ceil(subAchievementListBox:GetItemCount() / 2) * 80)
                    self.achievementsListBox:AddItem(categoryLabel)
                    self.achievementsListBox:AddItem(subAchievementListBox)
                end
            end
        end

        

        for i = 1, self.achievementsListBox:GetItemCount() do
            local subAchievementListBox = self.achievementsListBox:GetItem(i)
            if subAchievementListBox.AddItem then
                for j = 1, subAchievementListBox:GetItemCount() do
                    subAchievementListBox:GetItem(j):CheckVisibility(subAchievementListBox:GetTop())
                end
            end

        end
    end

    

    


end

function IntroWindow:SwapPages(pageName)

    for key, value in pairs(self.optionPages) do
        value:SetVisible(false)
    end

    if self.optionPages[pageName] then 
        self.optionPages[pageName]:SetVisible(true) 
    end
end

function GetAchievementsCategories()
    return {
        [1] = "General",
        [2] = "Class specific",
        [3] = "Hidden",
    }
end

-- 0x41091DEF chicken
-- 0x410F24E3

function IntroWindow:GetAchievementControl(ID, value, clickable, classSpecific, done)
    local width = 190 * 2
    local height = 80
    local borderSize = 1
    local achievementControl = Turbine.UI.Control()
    achievementControl:SetSize(width, height)
    achievementControl:SetBackColor(Turbine.UI.Color.Black) 
    achievementControl:SetMouseVisible(clickable)

    achievementControl.CheckVisibility = function(sender, listBoxHeight)
        if achievementControl:GetTop() + listBoxHeight < 0 or achievementControl:GetTop() + height + listBoxHeight > self.achievementsListBox:GetHeight() then
            achievementControl.Icon:SetVisible(false)
        else
            achievementControl.Icon:SetVisible(true)
        end 
    end

    achievementControl.Inner = Turbine.UI.Control()
    achievementControl.Inner:SetParent(achievementControl)
    achievementControl.Inner:SetSize(width - 2 * borderSize, height - 2 * borderSize)
    achievementControl.Inner:SetPosition(borderSize, borderSize)
    achievementControl.Inner:SetMouseVisible(false)

    if classSpecific == false then
        achievementControl.Inner:SetBackColor(Turbine.UI.Color.Gray) 
        achievementControl:SetMouseVisible(false)
    elseif classSpecific == nil or classSpecific then
        achievementControl.Inner:SetBackColor(Turbine.UI.Color.DarkRed)
        achievementControl:SetMouseVisible(true)
    end

    if not(clickable) then
        achievementControl.Inner:SetBackColor(Turbine.UI.Color.Teal) 
    end

    if done then
        achievementControl.Inner:SetBackColor(Turbine.UI.Color.Green) 
        achievementControl:SetMouseVisible(false)
    end

    achievementControl.IconBorder = Turbine.UI.Control()
    achievementControl.IconBorder:SetParent(achievementControl.Inner)
    achievementControl.IconBorder:SetSize(height - 2 * borderSize, height - 2 * borderSize )
    achievementControl.IconBorder:SetBackColor(Turbine.UI.Color.Black) 
    achievementControl.IconBorder:SetMouseVisible(false)

    achievementControl.Icon = Turbine.UI.Control()
    achievementControl.Icon:SetParent(achievementControl.IconBorder)
    achievementControl.Icon:SetBackground(value.Icon)
    achievementControl.Icon:SetStretchMode(2)
    achievementControl.Icon:SetStretchMode(1)
    achievementControl.Icon:SetSize(height - 2 * borderSize - 4, height - 2 * borderSize - 4)
    achievementControl.Icon:SetPosition(2,2)
    --achievementControl.Icon:SetBackColor(Turbine.UI.Color.Teal) 
    achievementControl.Icon:SetMouseVisible(false)



    achievementControl.Title = Turbine.UI.Label()
    achievementControl.Title:SetParent(achievementControl.Inner)
    achievementControl.Title:SetSize(achievementControl.Inner:GetWidth() - height, 30)
    achievementControl.Title:SetPosition(height, 0)
    achievementControl.Title:SetFontStyle(Turbine.UI.FontStyle.Outline)
    achievementControl.Title:SetOutlineColor(Turbine.UI.Color.Black)
    achievementControl.Title:SetFont(Turbine.UI.Lotro.Font.TrajanProBold22)
    achievementControl.Title:SetMultiline(false)
    achievementControl.Title:SetText(value.Title)
    achievementControl.Title:SetMouseVisible(false)

    achievementControl.Description = Turbine.UI.Label()
    achievementControl.Description:SetParent(achievementControl.Inner)
    achievementControl.Description:SetSize(achievementControl.Inner:GetWidth() - height, 50)
    achievementControl.Description:SetPosition(height, 30)
    achievementControl.Description:SetFontStyle(Turbine.UI.FontStyle.Outline)
    achievementControl.Description:SetOutlineColor(Turbine.UI.Color.Black)
    achievementControl.Description:SetFont(Turbine.UI.Lotro.Font.BookAntiqua16)
    achievementControl.Description:SetText(value.Description)
    achievementControl.Description:SetMouseVisible(false)

    achievementControl.Selected = false

    achievementControl.MouseClick = function()
        if not(achievementControl.Selected) then
            achievementControl.Inner:SetBackColor(Turbine.UI.Color.Green) 
            achievementControl.Selected = true
            self.achievements[value.Title] = ID
        else
            achievementControl.Inner:SetBackColor(Turbine.UI.Color.DarkRed) 
            achievementControl.Selected = false
            self.achievements[value.Title] = nil
        end
    end

    return achievementControl
end

function IntroWindow:GetRulesControl(parent)
    local rulesListBox = Turbine.UI.ListBox()
    rulesListBox:SetParent(parent)
    rulesListBox:SetSize(parent:GetWidth() - 10, 550)
    rulesListBox:SetPosition(0, 0)
    --rulesListBox:SetBackColor(Turbine.UI.Color.Red)
    rulesListBox:SetOrientation(Turbine.UI.Orientation.Vertical)

    local rulesListBoxScrollbar = Turbine.UI.Lotro.ScrollBar()
    rulesListBoxScrollbar:SetParent(parent)
    rulesListBoxScrollbar:SetSize(10, 550)
    rulesListBoxScrollbar:SetPosition(parent:GetWidth() - 10, 0)
    rulesListBoxScrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)

    rulesListBox:SetVerticalScrollBar(rulesListBoxScrollbar)

    --self.iconControl:SetBackColor(Turbine.UI.Color.Red)

    local rulesLabel = Turbine.UI.Label()
    rulesLabel:SetSize(rulesListBox:GetWidth(), 30)
    rulesLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    rulesLabel:SetOutlineColor(Turbine.UI.Color.Black)
    rulesLabel:SetFont(Turbine.UI.Lotro.Font.Verdana23)
    rulesLabel:SetText("Rules:")

    rulesListBox:AddItem(rulesLabel)

    local rulesDescriptionLabel = Turbine.UI.Label()
    rulesDescriptionLabel:SetSize(rulesListBox:GetWidth(), 370)
    rulesDescriptionLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    rulesDescriptionLabel:SetOutlineColor(Turbine.UI.Color.Black)
    rulesDescriptionLabel:SetFont(Turbine.UI.Lotro.Font.Verdana18)
    rulesDescriptionLabel:SetText([[
        - All profession allowed.
        - No restriction on traits.
        - You can use gear that you pick up or craft.
        - No trading.
        - No Auction House.
        - No Mailbox.
        - No grouping in open world. 
        - No ettenmoors.
        - No missions or delvings.
        - No swift travelling.
        - No return to X skills.
        - No milestones.
        - No hobbit present gear.
        - No shared storage.
        - No mithril travel.
        - No VIP reward items.
        - No experience tomes
        - No experience boosting item
        - You are only allowed to enter The Great Barrows, Garth Agarwhen, Fornost, Annuminas 
          and Angmar clusters
]])

    rulesListBox:AddItem(rulesDescriptionLabel)

    local instanceHeaderLabel = Turbine.UI.Label()
    instanceHeaderLabel:SetSize(rulesListBox:GetWidth(), 30)
    instanceHeaderLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    instanceHeaderLabel:SetOutlineColor(Turbine.UI.Color.Black)
    instanceHeaderLabel:SetFont(Turbine.UI.Lotro.Font.Verdana23)
    instanceHeaderLabel:SetText("Instance Rules:")
    
    rulesListBox:AddItem(instanceHeaderLabel)

    local instanceDescriptionLabel = Turbine.UI.Label()
    instanceDescriptionLabel:SetSize(rulesListBox:GetWidth(), 80)
    instanceDescriptionLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    instanceDescriptionLabel:SetOutlineColor(Turbine.UI.Color.Black)
    instanceDescriptionLabel:SetFont(Turbine.UI.Lotro.Font.Verdana18)
    instanceDescriptionLabel:SetText([[
        - Instance runs are authorized, but only 1 run of each instance.
        - Everyone in the party must be following hardcore rules.
        - Nobody can be over the maximum level of a instance.
        - You are only allowed to be grouped for and join an instance from Bree town.
]])

    rulesListBox:AddItem(instanceDescriptionLabel)

    local classHeaderLabel = Turbine.UI.Label()
    classHeaderLabel:SetSize(rulesListBox:GetWidth(), 30)
    classHeaderLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    classHeaderLabel:SetOutlineColor(Turbine.UI.Color.Black)
    classHeaderLabel:SetFont(Turbine.UI.Lotro.Font.Verdana23)
    classHeaderLabel:SetText("Class and item rules:")

    rulesListBox:AddItem(classHeaderLabel)
    
    local classDescriptionrLabel = Turbine.UI.Label()
    classDescriptionrLabel:SetSize(rulesListBox:GetWidth(), 90)
    classDescriptionrLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    classDescriptionrLabel:SetOutlineColor(Turbine.UI.Color.Black)
    classDescriptionrLabel:SetFont(Turbine.UI.Lotro.Font.Verdana18)
    classDescriptionrLabel:SetText([[
        - Hunters are not allowed to use desperate flight.
        - Minstrels are not allowed to use Still As Death.
        - Hobbits are not allowed to use Hobbit-silence.
        - No rezzing.
]])

    rulesListBox:AddItem(classDescriptionrLabel)

    local verificationHeaderLabel = Turbine.UI.Label()
    verificationHeaderLabel:SetSize(rulesListBox:GetWidth(), 30)
    verificationHeaderLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    verificationHeaderLabel:SetOutlineColor(Turbine.UI.Color.Black)
    verificationHeaderLabel:SetFont(Turbine.UI.Lotro.Font.Verdana23)
    verificationHeaderLabel:SetText("Verification:")

    rulesListBox:AddItem(verificationHeaderLabel)

    local verificationDescriptionLabel = Turbine.UI.Label()
    verificationDescriptionLabel:SetSize(rulesListBox:GetWidth(), 30)
    verificationDescriptionLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    verificationDescriptionLabel:SetOutlineColor(Turbine.UI.Color.Black)
    verificationDescriptionLabel:SetFont(Turbine.UI.Lotro.Font.Verdana18)
    verificationDescriptionLabel:SetText([[
        - Time played and plugin uptime need to be within a 10% margin
]])

    rulesListBox:AddItem(verificationDescriptionLabel)



    local partyHeaderLabel = Turbine.UI.Label()
    partyHeaderLabel:SetSize(rulesListBox:GetWidth(), 30)
    partyHeaderLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    partyHeaderLabel:SetOutlineColor(Turbine.UI.Color.Black)
    partyHeaderLabel:SetFont(Turbine.UI.Lotro.Font.Verdana23)
    partyHeaderLabel:SetText("Duos/Trios/sextet:")

    rulesListBox:AddItem(partyHeaderLabel)

    local partyDescriptionLabel = Turbine.UI.Label()
    partyDescriptionLabel:SetSize(rulesListBox:GetWidth(), 120)
    partyDescriptionLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    partyDescriptionLabel:SetOutlineColor(Turbine.UI.Color.Black)
    partyDescriptionLabel:SetFont(Turbine.UI.Lotro.Font.Verdana18)
    partyDescriptionLabel:SetText([[

    - You must not leave the same zone as eachother.
    - You must choose a combo that starts in the same starting location.
    - If one of you dies, the other(s) must die and the run is over.
    - You can trade your partner(s) found or crafted items including gold.
    - Multiboxing goes against the spirit of the hardcore challenge and is not allowed.
]])

    rulesListBox:AddItem(partyDescriptionLabel)

    return rulesListBox

end

function IntroWindow:GetPartyControl(parent)
    local rulesLabel = Turbine.UI.Label()
    rulesLabel:SetParent(parent)
    rulesLabel:SetSize(parent:GetWidth(), 30)
    rulesLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    rulesLabel:SetOutlineColor(Turbine.UI.Color.Black)
    rulesLabel:SetFont(Turbine.UI.Lotro.Font.Verdana23)
    rulesLabel:SetText("Rules for fellowships:")

    local rulesDescriptionLabel = Turbine.UI.Label()
    rulesDescriptionLabel:SetParent(parent)
    rulesDescriptionLabel:SetSize(parent:GetWidth(), 260)
    rulesDescriptionLabel:SetPosition(30, 30)
    rulesDescriptionLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    rulesDescriptionLabel:SetOutlineColor(Turbine.UI.Color.Black)
    rulesDescriptionLabel:SetFont(Turbine.UI.Lotro.Font.Verdana18)
    rulesDescriptionLabel:SetText([[
- For new runs all players must have the hardcore plugin.
- You must choose a combo that starts in the same starting zone (unless you travel to eachother at level 1).
- You must stay in the same zone together.
- You must be logged in at the same time even if not questing.
- You are soulbound and share one life. If one of you dies, the other(s) must fall on the sword and the run is over.
- You can trade any solo self found items or crafted items to each other including gold.
]])

    if self.inProgress then
        local party = self.handler:GetCharacterData().Party
        if party then
            local i = 1
            for key, value in pairs(party) do
                local fellowLabel = Turbine.UI.Label()
                fellowLabel:SetParent(parent)
                fellowLabel:SetSize(200, 20)
                fellowLabel:SetPosition(20, 290 + (i - 1) * 20)
                fellowLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
                fellowLabel:SetOutlineColor(Turbine.UI.Color.Black)
                fellowLabel:SetFont(Turbine.UI.Lotro.Font.Verdana18)
                fellowLabel:SetText("Fellow " .. i)
        
                local nameBox = Turbine.UI.Lotro.TextBox()
                nameBox:SetParent(parent)
                nameBox:SetSize(200, 20)
                nameBox:SetPosition(100, 290 + (i - 1) * 20)
                nameBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
                nameBox:SetOutlineColor(Turbine.UI.Color.Black)
                nameBox:SetFont(Turbine.UI.Lotro.Font.Verdana18)
                nameBox:SetEnabled(false)
                nameBox:SetText(key)
                i = i + 1
            end       
        end
    else
        for i = 1, 6 do
            local fellowLabel = Turbine.UI.Label()
            fellowLabel:SetParent(parent)
            fellowLabel:SetSize(200, 20)
            fellowLabel:SetPosition(20, 290 + (i - 1) * 20)
            fellowLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
            fellowLabel:SetOutlineColor(Turbine.UI.Color.Black)
            fellowLabel:SetFont(Turbine.UI.Lotro.Font.Verdana18)
            fellowLabel:SetText("Fellow " .. i)
    
            local nameBox = Turbine.UI.Lotro.TextBox()
            nameBox:SetParent(parent)
            nameBox:SetSize(200, 20)
            nameBox:SetPosition(100, 290 + (i - 1) * 20)
            nameBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
            nameBox:SetOutlineColor(Turbine.UI.Color.Black)
            nameBox:SetFont(Turbine.UI.Lotro.Font.Verdana18)
    
            local lockButton = Turbine.UI.Lotro.Button()
            lockButton:SetParent(parent)
            lockButton:SetSize(100, 20)
            lockButton:SetPosition(300, 290 + (i - 1) * 20)
            lockButton:SetText("Lock")
    
            local errorLabel = Turbine.UI.Label()
            errorLabel:SetParent(parent)
            errorLabel:SetSize(300, 20)
            errorLabel:SetPosition(420, 290 + (i - 1) * 20)
            errorLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
            errorLabel:SetOutlineColor(Turbine.UI.Color.Black)
            errorLabel:SetFont(Turbine.UI.Lotro.Font.Verdana18)
    
            lockButton.Click = function()
                if lockButton:GetText() == "Lock" then
                    local fellowName = nameBox:GetText():gsub("^%l", string.upper)
                    errorLabel:SetText("Locked in " .. tostring(fellowName))
                    lockButton:SetText("Unlock")
                    self.partyNames[fellowName] = true
                    nameBox:SetEnabled(false)
                else
                    errorLabel:SetText("")
                    lockButton:SetText("Lock")
                    nameBox:SetEnabled(true)
                    self.partyNames[nameBox:GetText()] = nil
                end
            end
        end
    end
    



end

function IntroWindow:GetVerificationControl(parent)
    self.verificationLabel = Turbine.UI.Label()
    self.verificationLabel:SetParent(parent)
    self.verificationLabel:SetPosition(100, 70)
    self.verificationLabel:SetSize(300, 20)
    self.verificationLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.verificationLabel:SetOutlineColor(Turbine.UI.Color.Black)
    self.verificationLabel:SetText("Press button to verify")

    local playedButton = Turbine.UI.Lotro.Quickslot()
    playedButton:SetParent(parent)
    playedButton:SetUseOnRightClick(true)
    --playedButton:SetShortcut()
    playedButton:SetMouseVisible(true)
    playedButton:SetSize(35, 35)
    playedButton:SetPosition(100, 100)
    --playedButton:SetWantsUpdates(true)
    playedButton:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Alias, "/played"))

    self.verificationStatusLabel = Turbine.UI.Label()
    self.verificationStatusLabel:SetParent(parent)
    self.verificationStatusLabel:SetPosition(100, 150)
    self.verificationStatusLabel:SetSize(300, 20)
    self.verificationStatusLabel:SetFont(Turbine.UI.Lotro.Font.Verdana23)
    self.verificationStatusLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.verificationStatusLabel:SetOutlineColor(Turbine.UI.Color.Black)
    self.verificationStatusLabel:SetMarkupEnabled(true)
    self.verificationStatusLabel:SetText("Verification Status")


    self.verificationGrade = Turbine.UI.Label()
    self.verificationGrade:SetParent(parent)
    self.verificationGrade:SetPosition(100, 180)
    self.verificationGrade:SetSize(200, 20)
    self.verificationGrade:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.verificationGrade:SetOutlineColor(Turbine.UI.Color.Black)
    self.verificationGrade:SetMarkupEnabled(true)
    --self.verificationGrade:SetFont(Turbine.UI.Lotro.Font.Verdana23)

    self.verificationValid = Turbine.UI.Label()
    self.verificationValid:SetParent(parent)
    self.verificationValid:SetPosition(100, 200)
    self.verificationValid:SetSize(200, 20)
    self.verificationValid:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.verificationValid:SetOutlineColor(Turbine.UI.Color.Black)
    --self.verificationValid:SetFont(Turbine.UI.Lotro.Font.Verdana23)
    self.verificationValid:SetMarkupEnabled(true)

    parent.VisibleChanged = function()
        local data = self.handler:GetCharacterData()
        if data.VerificationPlayedTime then
            if data.VerificationPlayedTime < 90 then
                self.verificationGrade:SetText(string.format("<rgb=#FF0000>%d%% => verification failed", data.VerificationPlayedTime))
            else
                self.verificationGrade:SetText(string.format("<rgb=#00FF00>%d%% => verification passed", data.VerificationPlayedTime))
            end
            if data.VerificationCheckTime + 1000 < Turbine.Engine.GetGameTime() then
                self.verificationValid:SetText("<rgb=#FF0000>Outdated")
            else
                self.verificationValid:SetText("<rgb=#00FF00>Up to date")
            end
        end
    end


end

function IntroWindow:SetVerification(totaltime, lastCheck)
    if totaltime then
        if totaltime < 90 then
            self.verificationGrade:SetText(string.format("<rgb=#FF0000>%d%% => verification failed", totaltime))
        else
            self.verificationGrade:SetText(string.format("<rgb=#00FF00>%d%% => verification passed", totaltime))
        end
        if lastCheck + 1000 < Turbine.Engine.GetGameTime() then
            self.verificationValid:SetText("<rgb=#FF0000>Outdated")
        else
            self.verificationValid:SetText("<rgb=#00FF00>Up to date")
        end
    end
end
